#  You may distribute under the terms of the GNU General Public License
#
#  (C) Paul Evans, 2008-2010 -- leonerd@leonerd.org.uk

package Circle::FE::Gtk;

use strict;
use warnings;

our $VERSION = '0.03';

1;

__END__

=head1 NAME

C<Circle::FE::Gtk> - L<Gtk2> frontend for the C<Circle> application host

=head1 AUTHOR

Paul Evans <leonerd@leonerd.org.uk>
